#!/usr/bin/env python3

# Class to read C6B files into Python.
# Copyright 2021 Andreas Nicolai.
# BSD (3) Licensed, use at your will.
# 
# You may want to extend the class to include writing functionality,
# see struct.pack().
# I would be happy to get such an extended Python script with writing 
# function back for distribution to other users. Please send them to
# andreas.nicolai -at- tu-dresden.de. Thanks!

import struct

class C6BReader:
	
	def __init__(self):
		pass
	
	def readString(self, fobj):
		# read number of characters to follow
		nchars = struct.unpack('i', fobj.read(4))[0]
		# read text
		text = fobj.read(nchars)
		return text.decode('UTF-8') # convert byte array to unicode/python string

	def readVector(self, fobj):
		# read number of doubles to follow
		nvalues = struct.unpack('i', fobj.read(4))[0]
		# read doubles
		values = []
		for i in range(nvalues):
			val = struct.unpack('d', fobj.read(8))[0]
			values.append(val)
		return values

	def read(self, filepath):
		try:
			fobj = open(filepath, "rb") # rb for "read binary"
			
			# magic header, 4 * 32bit integer
			first = fobj.read(4)
			second = fobj.read(4)
			
			if first != b'CLDF' or second != b'RLZ!':
				print("{} is not a C6B file.".format(filepath))
				return False
			
			version = struct.unpack('i', fobj.read(4))[0]
			print("C6B Version {}".format(version))
			dummy = struct.unpack('i', fobj.read(4))[0]
			if dummy != 0:
				print("{} has invalid magic header.".format(filepath))
				return False
			
			# read number of header lines
			headerLineCount = struct.unpack('i', fobj.read(4))[0]
			
			# read and display header lines
			for i in range(headerLineCount):
				# read a string in binary encoding
				line = self.readString(fobj)
				print(line)
				
			# now read data section
			
			self.Temperature = self.readVector(fobj) # in C
			self.RelativeHumidity = self.readVector(fobj) # in %
			self.DirectRadiationNormal = self.readVector(fobj) # in W/m2
			self.DiffuseRadiationHorizontal = self.readVector(fobj) # in W/m2
			self.WindDirection = self.readVector(fobj) # in Deg
			self.WindVelocity = self.readVector(fobj) # in m/s
			self.LongWaveCounterRadiation = self.readVector(fobj) # in W/m2
			self.AirPressure = self.readVector(fobj) # in Pa
			self.Rain = self.readVector(fobj) # in l/m2h
			
			# read optional time vector, if we have continuous data
			if len(self.Temperature) != 8760:
				self.TimePoints = self.readVector(fobj) # in s
			else:
				self.TimePoints = []
			return True
		except IOError as e:
			print(e)
			return False




# ** test **

reader = C6BReader()
reader.read("Bolzano_MN7.c6b")
